module Util.Chart where
import qualified Numeric
import qualified System.Process as Process
import qualified Graphics.Google.Chart as Chart
import qualified Util.Num as Num

import Ui

import qualified Perform.Signal as Signal


sig0 = Signal.signal [(0, 0), (1, 1), (2, 0)]
t0 = signal_to_url "hoho" (Signal.signal [(0, 0), (1, 1), (2, 0)])
t00 = data_vals sig0

open url = Process.system ("open '" ++ url ++ "'")

thin_sig = Signal.signal . thin . Signal.unpack
thin xs = case drop 3 xs of
    x : rest -> x : thin rest
    [] -> []

signals_to_url :: String -> [(String, Signal.Signal a)] -> String
signals_to_url title sigs = undefined

signal_to_url :: String -> Signal.Signal a -> String
signal_to_url title sig = Chart.chartURL $
    Chart.setSize 900 400 $
    Chart.setTitle title $
    add_data sig $
    Chart.setLegend ["sig1"] $
    Chart.newLineChart

add_data :: (Chart.AxisLabelChart c) => Signal.Signal a -> c -> c
add_data sig =
    Chart.setData (Chart.encodeDataExtended [vals])
    . Chart.setAxisRanges [(0, x_max), (0, y_max)]
    . Chart.setAxisLabels [x_labels, y_labels]
    . Chart.setAxisLabelPositions [x_positions, y_positions]
    . Chart.setAxisTypes [Chart.AxisBottom, Chart.AxisLeft]
    where
    ((x_max, y_max), (x_labels, y_labels), (x_positions, y_positions), vals) =
        data_vals sig

data_vals :: Signal.Signal a -> ((Int, Int), ([String], [String]),
    ([Int], [Int]), [Int])
data_vals sig = ((x_pos max_x, y_pos max_y), (x_labels, y_labels),
        (x_positions, y_positions), encode_extended max_y ys)
    where
    max_x = maximum (0 : xs)
    max_y = maximum (0 : ys)
    (xs, ys) = unzip $ Signal.unpack sig
    x_name (TrackPos x) = Numeric.showFFloat (Just 2) x ""
    y_name y = Numeric.showFFloat (Just 2) y ""
    x_pos (TrackPos x) = floor (x * 100)
    y_pos y = floor (y * 100)

    x_labels = map x_name xs
    x_positions = map x_pos xs

    num_ys = 10
    y_axis = [(i/num_ys) * max_y | i <- [0..num_ys]]
    y_labels = map y_name y_axis
    y_positions = map (y_pos . (*max_y)) y_axis

encode_extended :: Signal.Y -> [Signal.Y] -> [Int]
encode_extended max_y ys = map (extended_val max_y) ys

extended_val max_y val = floor $ Num.scale 0 4095 $ Num.normalize 0 max_y val
